//require base/PriorityQueue.js
//require base/bindEventFn.js

/**
Registers widget types and creates, destroys and lookups widget 
instances.

Expects the widget implementations to fullfill a (duck typed) interface.

WidgetImplementations need to register themselves with their logical name
and a function / namespace with the following functions/members 

var WidgetType=
{
name:            "ExampleWidget" // logical name of the widget type. Must be script name without extension in OpenSAGA.
createQuery:     "a.important",     // CSS query to execute to find all DOM elements that need to be initialized
destroyQuery:    null,            // CSS query for the destroy. if not present, the create query is used
create:
    function(DOMElement)
    {
        // code to initialize the widget with DOMElement being one of the queried elements.

        // Create can return an per-elem-object / javascript widget instance that is registered
        // This instance must have a "rootElem" property pointing to the root element for the widget
        // that must have an id.
        
        // if the instance object has a destroy function, this is called on destroy. 
    },
destroy:
    function(DOMElement)
    {
        // code to destroy the widget
    }      
}
*/

(function($)
{

    /** Stores all widget types */
    var widgets = {};
    /** Stores all widget instances */
    var widgetInstances = {};

    var widgetQueue = new PriorityQueue();

    var widgetCount = 0;

    var createWidgets = function(top, widgetType)
    {
        //console.debug("createWidgets %x, %o", top, widgetType);

        if (widgetType)
        {
            if (widgetType.createQuery)
            {
                elems = $(widgetType.createQuery,top);
                
                //console.debug("Widget %s = %o", name, elems);
                
                elen = elems.length;
                for (i=0; i < elen; i++)
                {
                    if (widgetType.create)
                    {
                        instance = widgetType.create(elems[i]);
                        if (instance)
                        {
                            widgetInstances[instance.rootElem.id] = instance;
                        }
                    }
                    else
                    {
                        console.error("no create function in widget type \"%s\"", widgetType.name);
                    }
                }
            }
            else
            {
                console.error("No createQuery property in widget type \"%s\"", widgetType.name);
            }
        }
    };
    
/**
Destroys the widgets with the given name in the
below the given element
*/
var destroyWidget = function(top,type)
{
    var widgetType = widgets[type];
    if (widgetType)
    {    
        var query = widgetType.destroyQuery || widgetType.createQuery;            
        var elems = $(query,top);
    
        var elen=elems.length;
        for (var i=0; i < elen; i++)
        {
            if (widgetType.destroy)
            {
                widgetType.destroy(elems[i]);
            }                
        }
    }
    else
    {
        console.warn("No widget definition named %s", type);
    }
};

/**
The Widget namespace
*/
this.Widget = { 
register:
    function(obj)
    {
        if (!obj)
        {
            throw new Error("no widget definition");
        }
        
        if (obj.priority === undefined)
        {
            obj.priority = 5;
        }
    
        var name = obj.name;
        widgets[name] = obj;
        
        
        widgetQueue.add(obj);
    },
/**
Creates all widgets below the given element or document.body if
no element is given. Also see reduceWidgetsTo. 
@param {DOMElement} top
*/
createWidgets:
    function(top, widgetNames)
    {
        top = top || document.body;
        var name,widgetType,elems,elen,instance,i;

        //console.debug("widgetQueue.objects = %o", widgetQueue.objects);

        if (widgetNames)
        {
            widgetQueue.callNamed(widgetNames, function()
            {
                createWidgets(top, this);
            });            
        }
        else
        {
            widgetQueue.call(function()
            {
                createWidgets(top, this);
            });            
        }
    },    
/**
Destroys all or some widget type below the given element.
You can specify the widget types to be destroyed as additional parameters.
if only one argument is given, all widget types are destroyed, 
@param {DOMElement} element top element. if not given, document.body is used.
@param {Array} widgetTypes widget types to be removed
*/
destroyWidgets:
    function(top, widgetTypes)
    {
        top = top || document.body;    
        widgetTypes = widgetTypes || widgets;
        
        //console.debug("destroying widgets of type %o in %x", widgetTypes, top);

        var name,i;

        for (name in widgets)
        {
            destroyWidget(top, name);
        }
        
        for (var id in widgetInstances)
        {
            widget = widgetInstances[id];

            if (opensaga.contains(top,widget.rootElem))
            {
                delete widgetInstances[id];
                if (widget.destroy)
                {
                    widget.destroy();
                }
            }
        }        
    },
lookup:
    function(id)
    {    
        return widgetInstances[id];
    },
instances:
    function()
    {
        return widgetInstances; 
    }
};
})(jQuery);

ScriptLoader.signalLoaded("base/Widget.js");
